//////////////////////////////////////////////////////////////////////////////
// exampleclient.h
// -------------------
// DeKorator window decoration for KDE
// -------------------
// Autor: moty rahamim
// Email: moty.rahamim@gmail.com
// Based on Example-0.8 by David Johnson <david@usermode.org>
//////////////////////////////////////////////////////////////////////////////

#ifndef EXAMPLECLIENT_H
#define EXAMPLECLIENT_H

#include <qbutton.h>
#include <kdecoration.h>
#include <kdecorationfactory.h>

class QSpacerItem;
class QPoint;

namespace DeKorator
{

class DeKoratorClient;

enum ButtonType {
    ButtonHelp = 0,
    ButtonMax,
    ButtonMin,
    ButtonClose,
    ButtonMenu,
    ButtonSticky,
    ButtonAbove,
    ButtonBelow,
    ButtonShade,
    ButtonTypeCount
};

// DeKoratorFactory /////////////////////////////////////////////////////////////

class DeKoratorFactory: public KDecorationFactory
{
public:
    DeKoratorFactory();
    virtual ~DeKoratorFactory();
    virtual KDecoration *createDecoration( KDecorationBridge *b );
    virtual bool reset( unsigned long changed );

    static bool initialized();
    static Qt::AlignmentFlags titleAlign();
    QPixmap midTitleBg_, leftFrameBg_, buttomFrameBg_, rightFrameBg_, topLeftCornerBg_, topRightCornerBg_, buttomLeftCornerBg_, buttomRightCornerBg_, leftTitleBg_, rightTitleBg_, leftButtonsBg_, rightButtonsBg_,
    buttonHelpPix_,
    buttonMaxPix_,
    buttonMinPix_,
    buttonClosePix_,
    buttonStickyPix_, buttonStickyDownPix_,
    buttonAbovePix_, buttonAboveDownPix_,
    ButtbonBelowPix_, ButtbonBelowDownPix_,
    buttonShadePix_, buttonShadeDownPix_;
    QBitmap topLeftCornerBitmap_, topRightCornerBitmap_, buttomLeftCornerBitmap_, buttomRightCornerBitmap_;
private:
    bool readConfig();
    void loadPixmaps();
    void colorizeDecoPixmaps();
    void colorizeButtonsPixmaps();

private:
    static bool initialized_;
    static Qt::AlignmentFlags titlealign_;
    static bool colorizeFrames_;
    static bool colorizeButtons_;
    static bool needInit_;
    static QString framesPath_;
    static QString buttonsPath_;
    static QString masksPath_;
public:
    static QString buttonsHoveType_;
    static float effectsAmount_;
    static bool useMasks_;

};

inline bool DeKoratorFactory::initialized()
{ return initialized_; }

inline Qt::AlignmentFlags DeKoratorFactory::titleAlign()
{ return titlealign_; }











// DeKoratorButton //////////////////////////////////////////////////////////////

class DeKoratorButton : public QButton
{
public:
    DeKoratorButton( bool isLeft, DeKoratorClient *parent = 0, const char *name = 0,
                   const QString &tip = NULL,
                   ButtonType type = ButtonHelp,
                   QPixmap *buttonPix = 0 );
    ~DeKoratorButton();

    void setPixmap( QPixmap *buttonPix = 0 );
    QSize sizeHint() const;
    ButtonState lastMousePress() const;
    void reset();

private:
    void enterEvent( QEvent *e );
    void leaveEvent( QEvent *e );
    void mousePressEvent( QMouseEvent *e );
    void mouseReleaseEvent( QMouseEvent *e );
    void drawButton( QPainter *painter );

private:
    DeKoratorClient *client_;
    ButtonType type_;
    QBitmap *deco_;
    QPixmap decoPix_;
    ButtonState lastmouse_;
    bool isLeft_;
    bool hover_;
};

inline Qt::ButtonState DeKoratorButton::lastMousePress() const
    { return lastmouse_; }

inline void DeKoratorButton::reset()
{ repaint( false ); }












// DeKoratorClient //////////////////////////////////////////////////////////////

class DeKoratorClient : public KDecoration
{
    Q_OBJECT
public:
    DeKoratorClient( KDecorationBridge *b, KDecorationFactory *f );
    virtual ~DeKoratorClient();

    virtual void init();

    virtual void activeChange();
    virtual void desktopChange();
    virtual void captionChange();
    virtual void iconChange();
    virtual void maximizeChange();
    virtual void shadeChange();

    virtual void borders( int &l, int &r, int &t, int &b ) const;
    virtual void resize( const QSize &size );
    virtual QSize minimumSize() const;
    virtual Position mousePosition( const QPoint &point ) const;

private:
    void addButtons( QBoxLayout* layout, const QString& buttons, bool isLeft );

    bool eventFilter( QObject *obj, QEvent *e );
    void mouseDoubleClickEvent( QMouseEvent *e );
    void paintEvent( QPaintEvent *e );
    void resizeEvent( QResizeEvent * );
    void showEvent( QShowEvent * );
    void doShape();

private slots:
    void maxButtonPressed();
    void menuButtonPressed();
    void aboveButtonPressed();
    void belowButtonPressed();
    void shadeButtonPressed();
    void keepAboveChange( bool );
    void keepBelowChange( bool );

private:
    DeKoratorButton *button[ ButtonTypeCount ];
    QSpacerItem *titlebar_;
public:
    DeKoratorFactory *decoFactory_;
};

} // namespace DeKorator

#endif // EXAMPLECLIENT_H
